package cz.mendelu.adaptive;

import java.security.Principal;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import cz.mendelu.adaptive.domain.Classroom;
import cz.mendelu.adaptive.services.ClassroomDaoService;
import cz.mendelu.adaptive.services.CustomSecurityUser;
import cz.mendelu.adaptive.services.ExamDaoService;
import cz.mendelu.adaptive.services.UserDaoService;

@Controller
public class ClassroomController {
		
	@Autowired
	public ExamDaoService examDaoService;

	@Autowired
	public UserDaoService userDaoService;
	
	@Autowired
	public ClassroomDaoService classroomDaoService;
	
	@RequestMapping(value = "/classroomView", method = RequestMethod.GET)
	public ModelAndView classroomView(Authentication auth) { 
		
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();

		return new ModelAndView("classroomView", "classes", classroomDaoService.findClassroomsByMemberId(userId));
	}

	@RequestMapping(value = "/classroomManage/{classId}", method = RequestMethod.GET)
	public ModelAndView classroomsManagementView(Principal principal, @PathVariable("classId") Long classId) {
		
		ModelAndView mv = new ModelAndView("classroomManage");
		mv.addObject("Exams", examDaoService.findPrivateClassroomExamsByEmail(principal.getName()));
		mv.addObject("Users", classroomDaoService.findClassroomUsersById(classId));
		mv.addObject("myExams", classroomDaoService.findClassroomExamsById(classId)); 
		
		return mv;
	}
	
	@RequestMapping(value = "/classroomDetails/{classId}", method = RequestMethod.GET)
	public ModelAndView classroomDetailView(Authentication auth, @PathVariable("classId") Long classId) {

		ModelAndView mv = new ModelAndView("classroomDetails");

		mv.addObject("Users", classroomDaoService.findClassroomUsersById(classId));
		mv.addObject("myExams", classroomDaoService.findClassroomExamsById(classId)); 
		
		return mv;
	}
	
	@RequestMapping(value = "/classroomOwned", method = RequestMethod.GET)
	public ModelAndView classroomsOwnedView(Authentication auth) {

		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();
		
		return new ModelAndView("classroomsOwned", "classes", classroomDaoService.findClassroomsByOwnerId(userId));
	}
	
	@RequestMapping(value = "/classroomCreate", method = RequestMethod.GET)
	public ModelAndView classroomCreateView() {

		return new ModelAndView("classroomCreate", "Classroom", new Classroom());
	}

	@RequestMapping(value = "/classroomEdit/{classId}", method = RequestMethod.GET)
	public ModelAndView classroomEditView(Authentication auth, @PathVariable("classId") Long classroomId) {
		return new ModelAndView("classroomEdit", "Classroom", classroomDaoService.findClassroomById(classroomId));
	}

	@RequestMapping(value = "/classroomManage/{classId}", method = RequestMethod.POST, params = "addexam")
	public String classroomManagementAddExam(@PathVariable("classId") Long classId, @RequestParam(required = true , value = "examId") Long examId) {
		
		classroomDaoService.addExamToClassroom(classId, examDaoService.findExamById(examId));
		return "redirect:/classroomManage/" + classId;
	}
	
	@RequestMapping(value = "/classroomManage/{classId}", method = RequestMethod.POST, params = "adduser")
	public String classroomManagementAddUser(@PathVariable("classId") Long classId, @RequestParam(required = true , value = "userId") Long userId) {
				
		classroomDaoService.addUserToClassroom(classId, userDaoService.findUserById(userId));
		return "redirect:/classroomManage/" + classId;
	}	
	
	@RequestMapping(value = "/classroomManage/{classId}", method = RequestMethod.POST, params = "removeExam")
	public String classroomManagementRemoveExam(@PathVariable("classId") Long classId, @RequestParam(required = true , value = "removeExam") Long examId) {
				
		classroomDaoService.removeExamFromClassroom(examId);
		return "redirect:/classroomManage/" + classId;
	}
	
	@RequestMapping(value = "/classroomRemove/{classId}", method = RequestMethod.GET)
	public String classroomAssignedRemoveUser(@PathVariable("classId") Long classId, Authentication auth) {
				
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();
		
		classroomDaoService.removeUserFromClassroom(classId, userId);
		return "redirect:/classroomView/";
	}	
	
	@RequestMapping(value = "/classroomManage/{classId}", method = RequestMethod.POST, params = "removeUser")
	public String classroomManagementRemoveUser(@PathVariable("classId") Long classId, @RequestParam(required = true , value = "removeUser") Long userId) {
				
		classroomDaoService.removeUserFromClassroom(classId, userId);
		return "redirect:/classroomManage/" + classId;
	}	
	
	@RequestMapping(value = "/classroomEdit/{classId}", method = RequestMethod.POST)
	public String classroomCreate(@ModelAttribute("Classroom") @Valid Classroom classroom) {
				
		classroomDaoService.updateClassroom(classroom);
		return "redirect:/classroomOwned/";
	}
	
	@RequestMapping(value = "/classroomDelete/{classroomId}", method = RequestMethod.GET)
	public String classroomRemove(Authentication auth, @PathVariable("classroomId") Long classroomId) {
				
		classroomDaoService.deleteClassroom(classroomId);
		return "redirect:/classroomOwned/";
	}
	
	@RequestMapping(value = "/classroomCreate", method = RequestMethod.POST)
	public String classroomCreate(Authentication auth, @ModelAttribute("Classroom") @Valid Classroom classroom) {
		
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();
		
		classroom.setOwner(userDaoService.findUserById(userId));
		classroomDaoService.createClassroom(classroom);
		return "redirect:/classroomManage/" + classroom.getClassroomId();
	}
	
	@RequestMapping(value = "/classroomResults/{classId}", method = RequestMethod.GET)
	public ModelAndView classroomResults(@PathVariable("classId") Long classId) {
		
		return new ModelAndView("classroomResults", "results", examDaoService.findExamResultsByClassroomId(classId));
	}
}
