package cz.mendelu.adaptive;

import java.io.IOException;
import java.security.Principal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.validation.Valid;

import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;
import org.codehaus.jackson.map.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cz.mendelu.adaptive.domain.Exam;
import cz.mendelu.adaptive.domain.ExamType;
import cz.mendelu.adaptive.domain.GraphChapterNode;
import cz.mendelu.adaptive.domain.Option;
import cz.mendelu.adaptive.domain.Question;
import cz.mendelu.adaptive.services.CustomSecurityUser;
import cz.mendelu.adaptive.services.ExamDaoService;
import cz.mendelu.adaptive.services.UserDaoService;




@Controller
public class ExamController {
	
	private static final Logger logger = LoggerFactory.getLogger(ExamController.class);
	
	
	
	@Autowired
	public ExamDaoService examDaoService;
	
	@Autowired
	public UserDaoService userDaoService;
	
	@RequestMapping(value = "/examsView", method = RequestMethod.GET)
	public ModelAndView examView(Principal principal) {
		logger.info("GET request on examsView page. ");
		ModelAndView mv = new ModelAndView("examsView");
		mv.addObject("myExams", examDaoService.findExamsByCreatorEmail(principal.getName()));
		
		return mv;
	
	}
	@RequestMapping(value = "/examsView", method = RequestMethod.POST)
	public String removeExam(Authentication auth, @RequestParam(required = true, value = "examId") Long examId) {	
		logger.info("GET request on examsView page. ");
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();
		
		if (!examDaoService.isUserExamCreator(userId, examId)) 
			return "403";
		examDaoService.removeExamById(examId);
		
		return "redirect:/examsView";
	}
	@RequestMapping(value = "/removeQuestion", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody void removeQuestion(@RequestBody String src) {
		ObjectMapper mapper = new ObjectMapper();
		
		try {
			examDaoService.removeQuestionByQuestionId(mapper.readValue(src, Long.class));
		} catch (JsonParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (JsonMappingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	@RequestMapping(value = "/showQuestion", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody String showQuestion(@RequestBody String src) {
		ObjectMapper mapper = new ObjectMapper();
		String response = null;
		try {	
			Question question = examDaoService.findQuestionById(mapper.readValue(src, Long.class));
			List<Option> list = examDaoService.findOptionsByQuestionId(mapper.readValue(src, Long.class));
			question.setOptionList(list);
			response = mapper.writeValueAsString(question);
		} catch (JsonParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (JsonMappingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return response;
	}

	@RequestMapping(value = "/createExam", method = RequestMethod.GET)
	public ModelAndView createExamView() {

		logger.info("GET request on createExam page. ");
		ModelAndView mv = new ModelAndView("createExam");

		mv.addObject("types", ExamType.values());
		mv.addObject("Exam", new Exam());
		return mv;
	}
	
	@RequestMapping(value = "/examEdit/{examId}", method = RequestMethod.GET)
	public ModelAndView editExamView(Authentication auth, @PathVariable("examId") Long examId) {
		
		logger.info("GET request on editExam page.");
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();
		
		if (!examDaoService.isUserExamCreator(userId, examId)) 
			return new ModelAndView("403");
		
		ModelAndView mv = new ModelAndView("editExam");
		mv.addObject("Exam", examDaoService.findExamById(examId));

		return mv;
	}

	@RequestMapping(value = "/examEdit/{examId}", method = RequestMethod.POST)
	public String editExam(@ModelAttribute("Exam") @Valid Exam exam, BindingResult result) {
		System.out.println(exam.getExamTitle());
		logger.info("POST request on examEdit page. " );
		
		if (result.hasErrors()) {
			for (ObjectError o : result.getAllErrors())
				System.out.println(o.getDefaultMessage());
			return "editExam";
		}
		exam.setCreatedOn(new Date());
		examDaoService.updateExam(exam);
		return "redirect:/examsView";
	}
	
	@RequestMapping(value = "/createExam", method = RequestMethod.POST)
	public String createExam(@RequestParam("userId") Long userId, @ModelAttribute("Exam") @Valid Exam exam, BindingResult result) {
		
		logger.info("POST request on createExam page. " );
		exam.setCreator(userDaoService.findUserById(userId));
		if (result.hasErrors()) return "createExam";
		exam.setCreatedOn(new Date());
		examDaoService.createExam(exam);
		
		return "redirect:/addExamContent/" + exam.getExamId() + "/" + exam.getRootNode().getChapterId();
	}

	@RequestMapping(value = "/addExamContent/{examId}/{chapterId}", method = RequestMethod.GET)
	public ModelAndView manageExamView(Authentication auth, @PathVariable("examId") Long examId, @PathVariable("chapterId") Long chapterId, Model model) {
		logger.info("GET request on addExamContent page.");
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();
	
		GraphChapterNode current = examDaoService.findChapterById(chapterId);

		if (current == null || !examDaoService.isUserExamCreator(userId, examId) || !current.getExam().getExamId().equals(examId)) 
			return new ModelAndView("403");
		
		ModelAndView mv = new ModelAndView("addExamContent");
		mv.addObject("Question", new Question());
		if (!model.containsAttribute("chapterForm")) mv.addObject("chapterForm", new GraphChapterNode());
		mv.addObject("questions", examDaoService.findQuestionsByChapterId(chapterId));
		mv.addObject("current",  current);

		return mv;
	}
	
	@RequestMapping(value = "/navigateChapters/{examId}/{chapterId}", method = RequestMethod.GET)
	public ModelAndView manageExamNavigationView(Authentication auth, @PathVariable("examId") Long examId, @PathVariable("chapterId") Long chapterId) {
		logger.info("GET request on createQuestion page.");
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();

		if (!examDaoService.isUserExamCreator(userId, examId)) 
			return new ModelAndView("403");
			
		ModelAndView mv = new ModelAndView("navigateChapters");
		mv.addObject("chapters", examDaoService.findAllExamChaptersByExamId(examId));
		mv.addObject("parents", examDaoService.findFirstAncestorsById(chapterId));
		mv.addObject("current", examDaoService.findChapterById(chapterId));
		mv.addObject("children", examDaoService.findFirstChapterSuccessorsByParentId(chapterId));
		return mv;
	}
		
	@RequestMapping(value = "/addExamContent/{examId}/{chapterId}", method = RequestMethod.POST, params="option")
	public String editQuestion(@RequestParam("editDescr") String description, 
			@RequestParam("questionId") Long questionId,
			@RequestParam("editLabel") String label,
			@RequestParam("editId") List<Long> ids, 
			@RequestParam("editText") List<String> optionTexts, 
			@RequestParam("correctId") Long correctId, 
			@PathVariable("examId") Long examId, 
			@PathVariable("chapterId") Long chapterId) {
		logger.info("POST request on addExamContent page.");
		
		List<Option> editedOptions = new ArrayList<Option>();
		for (int i = 0; i < ids.size(); i++) {
			if (ids.get(i).equals(correctId)){
				editedOptions.add(new Option(ids.get(i), optionTexts.get(i), true));
			}else{
				editedOptions.add(new Option(ids.get(i), optionTexts.get(i), false));
			}
		}
		Question question = examDaoService.findQuestionById(questionId);
		question.setOptionList(editedOptions);
		if (label != null) question.setLabel(label);
		if (description != null) question.setDescription(description);
		examDaoService.updateQuestion(question);
		return "redirect:/addExamContent/" + examId + "/" + chapterId;
	}
	
	@RequestMapping(value = "/addExamContent/{examId}/{chapterId}", method = RequestMethod.POST, params="chapter")
	public String addChapter(RedirectAttributes redirectAttributes, @PathVariable("examId") Long examId, @PathVariable("chapterId") Long chapterId, @ModelAttribute("chapterForm") @Valid GraphChapterNode chapterForm, BindingResult result) {
		logger.info("POST request on createChapter page.");
		if (result.hasErrors()) {
			redirectAttributes.addFlashAttribute("org.springframework.validation.BindingResult.chapterForm", result);
			redirectAttributes.addFlashAttribute("chapterForm", chapterForm);
			return "redirect:/addExamContent/" + examId + "/" + chapterId + "#chapter";
		}
		chapterForm.setChapterId(null);
		examDaoService.addSubChapter(chapterForm, chapterId);
		return "redirect:/addExamContent/" + examId + "/" + chapterId + "#chapter";
	}
	@RequestMapping(value = "/chapterEdit/{examId}/{chapterId}", method = RequestMethod.GET)
	public ModelAndView chapterEditView(Authentication auth, @PathVariable("examId") Long examId, @PathVariable("chapterId") Long chapterId, Model model){
		logger.info("GET request on editChapter page.");
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();

		if (!examDaoService.isUserExamCreator(userId, examId)) 
			return new ModelAndView("403");
			
		ModelAndView mv = new ModelAndView("chapterEdit");
		if (!model.containsAttribute("chapterForm")) mv.addObject("chapterForm", examDaoService.findChapterById(chapterId));
		mv.addObject("dependencies", examDaoService.findDependenciesByChapterId(chapterId));
		mv.addObject("dependebles", examDaoService.findDependableChaptersById(chapterId, examId));
		return mv;
	}
	@RequestMapping(value = "/chapterEdit/{examId}/{dependentId}", method = RequestMethod.POST, params="dependency")
	public String addDependancy(@RequestParam(required = true, value = "parentId") Long parentId, @PathVariable("examId") Long examId, @PathVariable("dependentId") Long dependentId) {
		logger.info("POST request on adddependency page.");
		examDaoService.addDependancy(parentId, dependentId);
		return "redirect:/chapterEdit/" + examId + "/" + dependentId;
	}
	
	@RequestMapping(value = "/chapterEdit/{examId}/{dependentId}", method = RequestMethod.POST, params="dependencyId")
	public String removeDependancy(@RequestParam(required = true, value = "dependencyId") Long dependencyId, @PathVariable("examId") Long examId, @PathVariable("dependentId") Long dependentId) {
		logger.info("POST request on removedependency page.");
		examDaoService.removeDependency(dependentId, dependencyId);
		return "redirect:/chapterEdit/" + examId + "/" + dependentId + "#dependencies";
	}
	
	@RequestMapping(value = "/chapterEdit/{examId}/{chapterId}", method = RequestMethod.POST, params="delete")
	public String removeChapter(@PathVariable("examId") Long examId, @PathVariable("chapterId") Long chapterId) {
		logger.info("POST request on deleteChapter page.");
		examDaoService.removeChapterById(chapterId);
		Long redirectId = examDaoService.findExamById(examId).getRootNode().getChapterId();
		return "redirect:/chapterEdit/" + examId + "/" + redirectId;
	}
	
	@RequestMapping(value = "/chapterEdit/{examId}/{chapterId}", method = RequestMethod.POST, params="edit")
	public String editChapter(RedirectAttributes redirectAttributes, @PathVariable("examId") Long examId, @PathVariable("chapterId") Long chapterId, @ModelAttribute("chapterForm") @Valid GraphChapterNode chapterForm, BindingResult result) {
		logger.info("POST request on editChapter page.");
		
		if (result.hasErrors()) {
			redirectAttributes.addFlashAttribute("org.springframework.validation.BindingResult.chapterForm", result);
			redirectAttributes.addFlashAttribute("chapterForm", chapterForm);
			return "redirect:/chapterEdit/" + examId + "/" + chapterId;
		}
		
		examDaoService.updateChapterTitle(chapterForm);
		return "redirect:/chapterEdit/" + examId + "/" + chapterId;
	}
	
	@RequestMapping(value = "/addExamContent/{examId}/{chapterId}", method = RequestMethod.POST, params="question")
	public String addQuestion(@ModelAttribute("Question") @Valid Question question, @PathVariable("examId") Long examId, @PathVariable("chapterId") Long chapterId) {
		logger.info("POST request on createChapter page.");
		examDaoService.addQuestionToChapter(question, chapterId);
		
		return "redirect:/addExamContent/" + examId + "/" + chapterId;
	}
}
