package cz.mendelu.adaptive;

import java.io.IOException;
import java.util.Arrays;
import java.util.Set;




import javax.validation.Valid;

import org.codehaus.jackson.JsonParseException;
import org.codehaus.jackson.map.JsonMappingException;
import org.codehaus.jackson.map.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import cz.mendelu.adaptive.domain.Profile;
import cz.mendelu.adaptive.domain.User;
import cz.mendelu.adaptive.domain.UserType;
import cz.mendelu.adaptive.services.CustomSecurityUser;
import cz.mendelu.adaptive.services.UserDaoService;
import cz.mendelu.adaptive.tools.MailProcessor;
@Controller
public class UserController {

	private static final Logger logger = LoggerFactory.getLogger(UserController.class);
	
	@Autowired
	public UserDaoService userDaoService;
	
	@RequestMapping(value = "/findUserAjax", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
	public @ResponseBody String findUsers(@RequestParam("nick") String nick) {
		ObjectMapper mapper = new ObjectMapper();
		String response = "";
		try {
			response = mapper.writeValueAsString(userDaoService.findClassroomUsersByNick(nick));
		} catch (JsonParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (JsonMappingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return response;
	}
	
	@RequestMapping(value = "/findUser", method = RequestMethod.GET)	
	public String findUserView() {
		return "findUser";
	}
	
	@RequestMapping(value = "/findUser", method = RequestMethod.GET, params = "uid")	
	public ModelAndView findUserByUID(@RequestParam("data") Long uid) {
		return new ModelAndView("findUser", "Users", Arrays.asList(userDaoService.findUserById(uid)));
	}
	
	@RequestMapping(value = "/findUser", method = RequestMethod.GET, params = "nick")	
	public ModelAndView findUserByNick(@RequestParam("data") String nick) {
		return new ModelAndView("findUser", "Users", userDaoService.findUsersByNick(nick));
	}
	
	@RequestMapping(value = "/findUser", method = RequestMethod.GET, params = "email")	
	public ModelAndView findUserByEmail(@RequestParam("data") String email) {
		return new ModelAndView("findUser", "Users", Arrays.asList(userDaoService.findUserByEmail(email)));	
	}
	@RequestMapping(value = "/findUser", method = RequestMethod.GET, params = "school")	
	public ModelAndView findUserBySchool(@RequestParam("data") String school) {
		return new ModelAndView("findUser", "Users", userDaoService.findUsersBySchool(school));
	}
	@RequestMapping(value = "/findUser", method = RequestMethod.POST)	
	public String removeUserView(Authentication auth, @RequestParam("user") Long userId) {
		Set<GrantedAuthority> roles = (Set<GrantedAuthority>) ((CustomSecurityUser) auth.getPrincipal()).getAuthorities();
		for (GrantedAuthority grantedAuthority : roles) 
			if (!grantedAuthority.getAuthority().toString().matches("ROLE_ADMIN")) 
				return "403";
		userDaoService.removeUserById(userId);
		return "findUser";
	}
	
	@RequestMapping(value = "/login", method = RequestMethod.GET)
	public ModelAndView loginView(@RequestParam(required = false) String err) {
		String msg = "";
		logger.info("GET request on Login page!");
		if (err != null) msg = "Invalid username or password, try again !"; 
		return new ModelAndView("login", "msg", msg);
	}
	
	@RequestMapping(value = "/403", method = RequestMethod.GET)
	public String unathorized() {
		logger.info("GET request on 403 page. ");
		return "403";
	
	}
	
	@RequestMapping(value = "/profileView/{userId}", method = RequestMethod.GET)
	public ModelAndView userProfileView(@PathVariable("userId") Long userId) {
		logger.info("GET request on userProfileView page. ");
		ModelAndView mv = new ModelAndView("profileView");
		mv.addObject("Profile", userDaoService.findProfile(userId));
		mv.addObject("User", userDaoService.findUserById(userId));
		return mv;
	}

	@RequestMapping(value = "/profileEdit", method = RequestMethod.GET)
	public ModelAndView profileView(Authentication auth, Model model) {
		logger.info("GET request on profile page. ");
		Long userId = ((CustomSecurityUser) auth.getPrincipal()).getId();
		ModelAndView mv = new ModelAndView("profileEdit");
		if (!model.containsAttribute("Profile")) mv.addObject("Profile", userDaoService.findProfile(userId));
		mv.addObject("User", userDaoService.findUserById(userId));
		mv.addObject("types", UserType.values());
		return mv;
	}
	
	@RequestMapping(value = "/profileEdit", method = RequestMethod.POST)
	public String profileEdit(RedirectAttributes redirectAttributes, @ModelAttribute("Profile") @Valid Profile profile, BindingResult result) {
		logger.info("POST request on profile page. ");
		if (result.hasErrors()){ 
			redirectAttributes.addFlashAttribute("org.springframework.validation.BindingResult.Profile", result);
			redirectAttributes.addFlashAttribute("Profile", profile);
			return "redirect:/profileEdit";
		}
		userDaoService.updateProfile(profile);
		return "redirect:/profileEdit";
	}
	
	@RequestMapping(value = "/register", method = RequestMethod.GET)
	public ModelAndView registerView() {
		logger.info("GET request on register page. ");
		ModelAndView mv = new ModelAndView("register");
		mv.addObject("User", new User());
		return mv;
	}
	
	@RequestMapping(value = "/register", method = RequestMethod.POST)
	public String register(@ModelAttribute("User") @Valid User user, BindingResult result) {
		if (result.hasErrors()) return "register";
		logger.info("POST request on register page. ");
		userDaoService.createUser(user);
		new MailProcessor(user.getEmail(), user.getNickName(), "Registration",  user.getEmail(), user.getPassword());
		return "redirect:/login";
	
	}
}
