package cz.mendelu.adaptive.dao.impl;

import java.util.Arrays;
import java.util.List;

import javax.annotation.Resource;

import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import cz.mendelu.adaptive.dao.ChapterDao;
import cz.mendelu.adaptive.domain.GraphChapterNode;


@Repository
@Transactional
public class ChapterDaoImpl implements ChapterDao{

	@Resource
	private SessionFactory sessionFactory;

	@SuppressWarnings("unchecked")
	@Override
	public List<GraphChapterNode> findFirstChapterSuccessorsByParentId(Long parentId) {
		return (List<GraphChapterNode>) sessionFactory.getCurrentSession().createQuery("SELECT cs FROM GraphChapterNode c JOIN c.successors cs WHERE c.chapterId =" + parentId).list();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<GraphChapterNode> findAllExamChaptersByExamId(Long examId) {
		return (List<GraphChapterNode>) sessionFactory.getCurrentSession().createQuery("SELECT c FROM GraphChapterNode c JOIN c.exam ce WHERE ce.examId =" + examId).list();
	}

	@Override
	public void removeChapterById(Long chapterId) {

		sessionFactory.getCurrentSession().createSQLQuery("DELETE FROM learning.nodesuccessors s WHERE s.successors_chapterid = " + chapterId).executeUpdate();
		sessionFactory.getCurrentSession().createQuery("UPDATE GraphChapterNode SET exam = NULL WHERE chapterId = " + chapterId).executeUpdate();
		GraphChapterNode chapter = (GraphChapterNode) sessionFactory.getCurrentSession().get(GraphChapterNode.class, chapterId);
		sessionFactory.getCurrentSession().delete(chapter);

	}
	@Override
	public void addSubChapter(GraphChapterNode newNode, Long parentId) {
		
		GraphChapterNode node = (GraphChapterNode) sessionFactory.getCurrentSession().get(GraphChapterNode.class, parentId);
		node.addSuccessor(findFirstChapterSuccessorsByParentId(parentId), newNode);
		sessionFactory.getCurrentSession().update(node);	
	}

	@Override
	public GraphChapterNode findChapterById(Long id) {
		return (GraphChapterNode) sessionFactory.getCurrentSession().get(GraphChapterNode.class, id);
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<GraphChapterNode> findFirstAncestorsById(Long parentId) {
		return (List<GraphChapterNode>) sessionFactory.getCurrentSession().createQuery("SELECT ca FROM GraphChapterNode c JOIN c.ancestors ca WHERE c.chapterId =" + parentId).list();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<GraphChapterNode> findDependableChaptersById(Long chapterId, Long examId) {
		Query childrenSubquery = sessionFactory.getCurrentSession().createQuery("SELECT cs.chapterId FROM GraphChapterNode s JOIN s.successors cs WHERE s.exam.examId = " + examId + " AND s.chapterId = "+ chapterId );
		Query parentsSubquery = sessionFactory.getCurrentSession().createQuery("SELECT ca.chapterId FROM GraphChapterNode s JOIN s.ancestors ca WHERE s.exam.examId = " + examId + " AND s.chapterId = "+ chapterId);
		Query result = sessionFactory.getCurrentSession().createQuery("SELECT new GraphChapterNode(c.chapterId, c.chapterTitle) FROM GraphChapterNode c WHERE c.exam.examId = " + examId + " AND c.chapterId <>" + chapterId + " AND c.isRoot = false AND c.chapterId NOT IN (:children) AND c.chapterId NOT IN (:parents)" );
		result.setParameterList("parents", parentsSubquery.list().isEmpty() ? Arrays.asList(0L) : parentsSubquery.list());
		result.setParameterList("children", childrenSubquery.list().isEmpty() ? Arrays.asList(0L) : childrenSubquery.list());
		return (List<GraphChapterNode>) result.list();
	}

	@Override
	public void addDependancy(Long chapterId, Long dependantId) {
		sessionFactory.getCurrentSession().createSQLQuery("INSERT INTO learning.nodesuccessors VALUES (" + chapterId + "," + dependantId + ");").executeUpdate();
		sessionFactory.getCurrentSession().createSQLQuery("UPDATE learning.graphchapternode SET dependencycount=dependencycount+1 WHERE chapterid =" + dependantId).executeUpdate();
	}

	@Override
	public Boolean isChapterDependant(Long chapterId) {
		return (((Integer) sessionFactory.getCurrentSession().createQuery("SELECT count(ca) FROM GraphChapterNode c JOIN c.ancestors ca WHERE c.chapterId = " + chapterId).uniqueResult()) > 1);
	}

	@Override
	public Long getExamChapterCount(Long examId) {
		return (Long) sessionFactory.getCurrentSession().createQuery("SELECT count(ch) FROM GraphChapterNode ch WHERE ch.exam.examId = " + examId).uniqueResult();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<GraphChapterNode> findDependenciesByChapterId(Long chapterId) {
		return (List<GraphChapterNode>) sessionFactory.getCurrentSession().createQuery("SELECT a From GraphChapterNode c JOIN c.ancestors a WHERE c.chapterId = " + chapterId + " AND c.dependencyCount > 1 AND substring(c.position,0,(length(c.position))) <> a.position").list();
	}

	@Override
	public void updateChapterTitle(GraphChapterNode chapterForm) {
		sessionFactory.getCurrentSession().createQuery("UPDATE GraphChapterNode  SET chapterTitle = :title WHERE chapterId = " + chapterForm.getChapterId()).setParameter("title", chapterForm.getChapterTitle()) .executeUpdate();
		
	}

	@Override
	public void removeDependency(Long dependentId, Long dependencyId) {
		sessionFactory.getCurrentSession().createSQLQuery("DELETE FROM learning.nodesuccessors WHERE ancestors_chapterid = " + dependencyId + " AND successors_chapterid = " + dependentId).executeUpdate();
	}
}
