package cz.mendelu.adaptive.dao.impl;

import java.util.List;

import javax.annotation.Resource;

import org.hibernate.SessionFactory;
import org.springframework.stereotype.Repository;


import org.springframework.transaction.annotation.Transactional;

import cz.mendelu.adaptive.dao.ExamDao;
import cz.mendelu.adaptive.domain.Exam;
import cz.mendelu.adaptive.domain.ExamType;


@Repository
@Transactional
public class ExamDaoImpl implements ExamDao{

	@Resource
	private SessionFactory sessionFactory;	
	
	@Override
	public Exam findExamById(Long id) {
		return (Exam) sessionFactory.getCurrentSession().get(Exam.class, id);
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Exam> findExamsByCreatorEmail(String userLoginName) {
		return (List<Exam>) sessionFactory.getCurrentSession().createQuery("Select e FROM Exam e JOIN e.creator eu WHERE eu.email= :userEmail").setParameter("userEmail", userLoginName).list();
	}

	@Override
	public void removeExamById(Long examId) {
		sessionFactory.getCurrentSession().createQuery("UPDATE GraphChapterNode c SET c.exam = NULL WHERE c.exam.examId =" +examId).executeUpdate();
		Exam exam = (Exam) sessionFactory.getCurrentSession().get(Exam.class, examId);
		sessionFactory.getCurrentSession().createQuery("DELETE FROM ExamResult c WHERE c.exam.examId =" +examId).executeUpdate();
		sessionFactory.getCurrentSession().delete(exam.getRootNode());
		sessionFactory.getCurrentSession().delete(exam);
	}

	@Override
	public void createExam(Exam exam) {
		sessionFactory.getCurrentSession().save(exam);
		
	}

	@Override
	public Boolean isUserExamCreator(Long userId, Long examId) {
		if (userId == null || examId == null) return false;
		return (((Long) sessionFactory.getCurrentSession().createQuery("Select e.creator.id FROM Exam e WHERE e.examId =" + examId).uniqueResult()).equals(userId));
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Exam> findPrivateClassroomExamsByEmail(String principalLoginName) {
		return (List<Exam>) sessionFactory.getCurrentSession().createQuery("Select e FROM Exam e JOIN e.creator eu WHERE e not in (SELECT ec FROM Classroom c JOIN c.examList ec) AND eu.email= :userEmail AND e.examType=" + ExamType.PRIVATE.ordinal()).setParameter("userEmail", principalLoginName).list();

	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Exam> findAvailablePrivateExamsByUserId(Long userId) {
		return (List<Exam>) sessionFactory.getCurrentSession().createQuery("Select e FROM Classroom c JOIN c.examList e JOIN c.memberList m WHERE e.isReady = TRUE AND m.id =" + userId).list();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<Exam> findPublicExams() {
		return (List<Exam>) sessionFactory.getCurrentSession().createQuery("FROM Exam e WHERE e.isReady = TRUE AND e.examType =" + ExamType.PUBLIC.ordinal()).list();
	}

	@Override
	public void updateExam(Exam exam) {
		System.out.println(exam.getExamTitle() );
		sessionFactory.getCurrentSession().createQuery("UPDATE Exam SET examTitle = :examTitle, examType = :examType, generateQuestions = :generateQuestions, isReady = :isReady, requiredSuccess = :requiredSuccess, subjectTitle = :subjectTitle WHERE examId = :examId").setProperties(exam).executeUpdate();
	}

}
