package cz.mendelu.adaptive.domain;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;
import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;
@Entity
public class ChapterResult {
	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE)
	private Long id;
	
	@OneToOne
	private GraphChapterNode chapter;
	
	@OneToMany
	@Fetch(FetchMode.JOIN)
	@Cascade(CascadeType.ALL)
	private List<Answer> answerList = new ArrayList<Answer>();
	
	private Float result;
	
	public List<Answer> getCorrectAnswers(){
		List<Answer> correctAnswers = new ArrayList<Answer>();
		
		for (Answer answer : answerList)
			if (answer.getCorrect()) correctAnswers.add(answer);
		return correctAnswers;
	}

	public List<Answer> getIncorrectAnswers(){
		List<Answer> incorrectAnswers = new ArrayList<Answer>();
		
		for (Answer answer : answerList)
			if (!answer.getCorrect()) incorrectAnswers.add(answer);
		return incorrectAnswers;
	}

	public void addAnswer(Answer answer){
		this.answerList.add(answer);
		this.result = (getCorrectAnswers().size()+0F) / getAnswerList().size();
	}
	
	public List<Answer> getAnswerList() {
		return answerList;
	}
	public void setAnswerList(List<Answer> answerList) {
		this.answerList = answerList;
	}
	public Long getId() {
		return id;
	}
	public void setId(Long id) {
		this.id = id;
	}
	public GraphChapterNode getChapter() {
		return chapter;
	}
	public void setChapter(GraphChapterNode chapter) {
		this.chapter = chapter;
	}
	public Float getResult() {
		return result;
	}
	public void setResult(Float result) {
		this.result = result;
	}

	public ChapterResult(GraphChapterNode chapter, Answer answer) {
		this.chapter = chapter;
		addAnswer(answer);
	}

	public ChapterResult(GraphChapterNode chapter) {
		this.chapter = chapter;
	}
	public ChapterResult() {	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((chapter == null) ? 0 : chapter.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ChapterResult other = (ChapterResult) obj;
		if (chapter == null) {
			if (other.chapter != null)
				return false;
		} else if (!chapter.equals(other.chapter))
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		return true;
	}
	
}
