package cz.mendelu.adaptive.domain;


import java.util.Arrays;
import java.util.List;

import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;

import javax.validation.constraints.Size;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;
import org.springframework.validation.annotation.Validated;




@Entity
@Validated
public class GraphChapterNode {

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE)
	private Long chapterId;

	@ManyToMany
	@JoinTable(name = "NodeSuccessors")
	@Cascade(value = { CascadeType.ALL})
	private List<GraphChapterNode> successors;

	@ManyToMany(mappedBy= "successors")
	@Cascade(value = {CascadeType.SAVE_UPDATE})
	private List<GraphChapterNode> ancestors;

	@OneToOne
	@Cascade(value = { CascadeType.SAVE_UPDATE })
	private Exam exam;
	
	private Boolean isRoot = false;
	
	private Integer dependencyCount = 1;
	
	private String position;
	@Size(min = 4, message = "Root chapter must be at least 4 characters long!")
	private String chapterTitle;
	
	@OneToMany
	@Cascade(value = { CascadeType.ALL, CascadeType.DELETE})
	private List<Question> questionList;


	public Long getChapterId() {
		return chapterId;
	}

	public void setChapterId(Long chapterId) {
		this.chapterId = chapterId;
	}

	public List<GraphChapterNode> getSuccessors() {
		return successors;
	}

	public void setSuccessors(List<GraphChapterNode> successors) {
		this.successors = successors;
	}
	
	public void addSuccessor(List<GraphChapterNode> currentSuccessors, GraphChapterNode newSuccessor){
		newSuccessor.position = this.position + String.valueOf(Character.toChars('A'+ successors.size()));
		newSuccessor.setExam(this.exam);
		newSuccessor.ancestors = Arrays.asList(this);
		currentSuccessors.add(newSuccessor);
		this.successors = currentSuccessors;
	}
	
	public Exam getExam() {
		return exam;
	}

	public void setExam(Exam exam) {
		this.exam = exam;
	}

	public String getChapterTitle() {
		return chapterTitle;
	}

	public void setChapterTitle(String chapterTitle) {
		this.chapterTitle = chapterTitle;
	}

	public List<Question> getQuestionList() {
		return questionList;
	}

	public void setQuestionList(List<Question> questions) {
		this.questionList = questions;
	}

	public List<GraphChapterNode> getAncestors() {
		return ancestors;
	}

	public void setAncestors(List<GraphChapterNode> ancestors) {
		this.ancestors = ancestors;
	}
	public GraphChapterNode(Exam exam) {
		this.exam = exam;
		this.isRoot = true;
		this.setPosition("R");
	}
	public GraphChapterNode() {
		
	}
	public GraphChapterNode(String chapterTitle, Exam exam, boolean isRoot) { 
		this.exam = exam;
		this.isRoot = isRoot;
		this.chapterTitle = chapterTitle;
		this.setPosition("R");
	}
	
	public GraphChapterNode(Long chapterId, String chapterTitle) {
		this.chapterId = chapterId;
		this.chapterTitle = chapterTitle;
	}

	public Boolean getIsRoot() {
		return isRoot;
	}

	public Integer getDepth() {
		return position.length();
	}

	public void setPosition(String position) {
		this.position = position;
	}
		
	public boolean isDependant(){
		return this.dependencyCount > 1 ? true : false;
	}
	public Integer getDependencyCount() {
		return dependencyCount;
	}

	public String getPosition() {
		return position;
	}


	
	
}
