package cz.mendelu.adaptive.services;

import java.util.List;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;

import cz.mendelu.adaptive.dao.ChapterDao;
import cz.mendelu.adaptive.dao.ExamDao;
import cz.mendelu.adaptive.dao.QuestionDao;


import cz.mendelu.adaptive.dao.ResultDao;
import cz.mendelu.adaptive.domain.ChapterResult;
import cz.mendelu.adaptive.domain.Exam;
import cz.mendelu.adaptive.domain.ExamResult;
import cz.mendelu.adaptive.domain.GraphChapterNode;
import cz.mendelu.adaptive.domain.Option;
import cz.mendelu.adaptive.domain.Question;


@Service
public class ExamDaoService implements ExamDao, QuestionDao, ChapterDao, ResultDao{
	
	@Resource
	private ExamDao examDao;
	
	@Resource
	private ResultDao resultDao;
	
	@Resource
	private ChapterDao chapterNodeDao;
	
	@Resource
	private QuestionDao questionDao;
	
	@Override
	public Question findQuestionById(Long id) {
		return questionDao.findQuestionById(id);
	}

	@Override
	public List<Question> findQuestionsByChapterId(Long chapterId) {
		return questionDao.findQuestionsByChapterId(chapterId);
	}

	@Override
	public List<Option> findOptionsByQuestionId(Long questionId) {
		return questionDao.findOptionsByQuestionId(questionId);
	}

	@Override
	public void removeQuestionByQuestionId(Long questionId) {
		questionDao.removeQuestionByQuestionId(questionId);
		
	}

	@Override
	public void addQuestionToChapter(Question question, Long chapterId) {
		questionDao.addQuestionToChapter(question, chapterId);
		
	}

	@Override
	public List<GraphChapterNode> findFirstChapterSuccessorsByParentId(Long parentId) {
		return chapterNodeDao.findFirstChapterSuccessorsByParentId(parentId);
	}

	@Override
	public List<GraphChapterNode> findAllExamChaptersByExamId(Long examId) {
		return chapterNodeDao.findAllExamChaptersByExamId(examId);
	}


	@Override
	public Exam findExamById(Long id) {
		return examDao.findExamById(id);
	}

	@Override
	public void removeExamById(Long examId) {
		examDao.removeExamById(examId);
	}

	@Override
	public void createExam(Exam exam) {
		examDao.createExam(exam);
	}

	@Override
	public void addSubChapter(GraphChapterNode newNode, Long parentId) {
		chapterNodeDao.addSubChapter(newNode, parentId);
		
	}

	@Override
	public GraphChapterNode findChapterById(Long id) {
		return chapterNodeDao.findChapterById(id);
	}

	@Override
	public List<GraphChapterNode> findFirstAncestorsById(Long parentId) {
		return chapterNodeDao.findFirstAncestorsById(parentId);
	}

	@Override
	public void updateOptions(List<Option> optionList) {
		questionDao.updateOptions(optionList);
		
	}

	@Override
	public List<GraphChapterNode> findDependableChaptersById(Long chapterId, Long examId) {
		return chapterNodeDao.findDependableChaptersById(chapterId, examId);
	}

	@Override
	public void addDependancy(Long chapterId, Long dependantId) {
		chapterNodeDao.addDependancy(chapterId, dependantId);
		
	}

	@Override
	public Boolean isUserExamCreator(Long userId, Long examId) {
		return examDao.isUserExamCreator(userId, examId);
	}

	@Override
	public List<Exam> findExamsByCreatorEmail(String userLoginName) {
		return examDao.findExamsByCreatorEmail(userLoginName);
	}

	public List<Exam> findPrivateClassroomExamsByEmail(String userLoginName) {
		return examDao.findPrivateClassroomExamsByEmail(userLoginName);
	}

	@Override
	public Boolean isChapterDependant(Long chapterId) {
		return chapterNodeDao.isChapterDependant(chapterId);
	}

	@Override
	public Boolean isOptionCorrectByOptionId(Long opionId) {
		return questionDao.isOptionCorrectByOptionId(opionId);
	}

	@Override
	public List<Exam> findAvailablePrivateExamsByUserId(Long userId) {
		return examDao.findAvailablePrivateExamsByUserId(userId);
	}

	@Override
	public List<Exam> findPublicExams() {
		return examDao.findPublicExams();
	}

	@Override
	public Long getExamChapterCount(Long examId) {
		return chapterNodeDao.getExamChapterCount(examId);
	}

	@Override
	public void updateQuestion(Question question) {
		questionDao.updateQuestion(question);
		
	}

	public List<GraphChapterNode> findDependenciesByChapterId(Long chapterId) {
		return chapterNodeDao.findDependenciesByChapterId(chapterId);
	}

	public void updateChapterTitle(GraphChapterNode chapterForm) {
		 chapterNodeDao.updateChapterTitle(chapterForm);
		
	}

	@Override
	public void removeChapterById(Long chapterId) {
		chapterNodeDao.removeChapterById(chapterId);
		
	}

	@Override
	public void createExamResult(ExamResult examResult) {
		resultDao.createExamResult(examResult);	
	}

	@Override
	public ExamResult findExamResultByResultId(Long resultId) {
		return resultDao.findExamResultByResultId(resultId);
	}

	@Override
	public List<ChapterResult> findChapterResultByUserId(Long userId, Long examResultId) {
		return resultDao.findChapterResultByUserId(userId, examResultId);
	}

	@Override
	public void removeExamResult(Long resultId) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public List<ExamResult> findExamResultsByOwnerId(Long userId) {
		return resultDao.findExamResultsByOwnerId(userId);
	}

	@Override
	public List<ExamResult> findExamResultsByUserId(Long userId) {
		return resultDao.findExamResultsByUserId(userId);
	}

	public void updateExam(Exam exam) {
		examDao.updateExam(exam);
	}

	@Override
	public List<Question> findRandomQuestionsByChapterId(Long chapterId,
			int quantity) {
		return questionDao.findRandomQuestionsByChapterId(chapterId, quantity);
	}
	@Override
	public void removeDependency(Long dependentId, Long dependencyId) {
		chapterNodeDao.removeDependency(dependentId, dependencyId);
		
	}

	@Override
	public List<ExamResult> findExamResultsByClassroomId(Long classroomId) {
		return resultDao.findExamResultsByClassroomId(classroomId);
	}
}
