package cz.mendelu.adaptive.tools;

import java.util.ArrayList;
import java.util.List;

import cz.mendelu.adaptive.domain.Answer;
import cz.mendelu.adaptive.domain.ChapterResult;
import cz.mendelu.adaptive.domain.Exam;
import cz.mendelu.adaptive.domain.GraphChapterNode;
import cz.mendelu.adaptive.domain.Question;
import cz.mendelu.adaptive.domain.ExamResult;
import cz.mendelu.adaptive.domain.User;



class ActiveExam {

	private User user;
	
	private Exam exam;
	
	private List<GraphChapterNode> chapterQueue = new ArrayList<GraphChapterNode>();
	
	private ExamResult result;
	
	private List<Question> questionQueue = new ArrayList<Question>();
	
	public void addChapterAnswer(GraphChapterNode chapter, Answer answer) {
		result.addChapterResultAnswer(chapter, answer);
	}
	
	public Boolean isChapterAnsweredCorrectly(GraphChapterNode chapter){
		for  (ChapterResult res : result.getChapterResults()){
			if (res.getChapter().getChapterId() == chapter.getChapterId()) {
				return (res.getCorrectAnswers().size()/(res.getAnswerList().size()+0F)*100)>=exam.getRequiredSuccess();
			}
				
		}
		return false;
	}
	
	public Boolean hasDoneDependencies(List<GraphChapterNode> chapters){
		int counter = 0;
		for (GraphChapterNode chapter : chapters)
			for (ChapterResult res : result.getChapterResults()){
				if (res.getChapter().getChapterId() == chapter.getChapterId())
					if (isChapterAnsweredCorrectly(chapter)) {
						counter++;
						break;
					}
			}
			return (chapters.size() == counter);
	}
	
	public ActiveExam(User user, Exam exam, List<Question> rootQuestions, GraphChapterNode rootChapter) {
		this.result = new ExamResult(user, exam);
		this.setUser(user);
		this.setExam(exam);
		this.chapterQueue.add(rootChapter);
		this.questionQueue = rootQuestions;
	}
	
	public void addQuestionsToQueue(List<Question> questions) {
		questionQueue.addAll(questions);
	}
		
	public Boolean isQuestionQueueEmpty() {
		return this.questionQueue.isEmpty();
	}
	
	public Boolean isChapterQueueEmpty() {
		return this.chapterQueue.isEmpty();
	}

	public void addChaptersToQueue(List<GraphChapterNode> chapter) {
		if (chapterQueue.isEmpty())
			chapterQueue.addAll(0, chapter);
		else{
			List<GraphChapterNode> addList = new ArrayList<GraphChapterNode>();
			for (GraphChapterNode ch : chapter){
				boolean exist = false;
				for (GraphChapterNode chQ : chapterQueue){
					if (chQ.getChapterId() == ch.getChapterId())
						exist = true;
				}
				if (exist == false)
					addList.add(ch);
			}
			chapterQueue.addAll(0, addList);
		}
	}
	
	public void removeChapterFromQueue() {
		this.chapterQueue.remove(0);
	}
	public int getGenerateCount(){
		return this.exam.getGenerateQuestions();
	}
	public void removeQuestionFromQueue(){
		this.questionQueue.remove(0);
	}
	public void removeDependentChapter(GraphChapterNode chapter){
		for (GraphChapterNode ch : chapterQueue)
			if (ch.getChapterId() == chapter.getChapterId()) {
				chapterQueue.remove(ch);
			}
	}
	public GraphChapterNode getChapter() {
		return chapterQueue.get(0);
	}	
	public Question getQuestion(){
		return questionQueue.get(0);
	}

	public Exam getExam() {
		return exam;
	}

	public void setExam(Exam exam) {
		this.exam = exam;
	}

	public ExamResult getResult() {
		return result;
	}

	public User getUser() {
		return user;
	}

	public void setUser(User user) {
		this.user = user;
	}
	
}
