package cz.mendelu.adaptive.tools;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import cz.mendelu.adaptive.domain.Answer;
import cz.mendelu.adaptive.domain.ChapterResult;
import cz.mendelu.adaptive.domain.Exam;
import cz.mendelu.adaptive.domain.ExamResult;
import cz.mendelu.adaptive.domain.GraphChapterNode;
import cz.mendelu.adaptive.domain.Question;
import cz.mendelu.adaptive.domain.User;


public class ActiveExamsRepository {

	private static Set<ActiveExam> activeExams = new HashSet<ActiveExam>();
	
	public static void addActiveExam(User user, Exam exam, List<Question> rootQuestions, GraphChapterNode rootChapter) {
		activeExams.add(new ActiveExam(user, exam, rootQuestions, rootChapter));
	}
	
	public static void removeActiveExam(Long userId) {
		for (ActiveExam exam : activeExams)
			if (exam.getUser().getId().equals(userId)) activeExams.remove(exam);
	}
	
	public static Boolean isChapterAnsweredCorrectly(Long userId, GraphChapterNode chapter) {
		return getActiveExam(userId).isChapterAnsweredCorrectly(chapter);
	}
		
	public static void removeChapter(Long userId) {
		getActiveExam(userId).removeChapterFromQueue();
	}
	
	public static void removeDependentChapter(Long userId, GraphChapterNode chapter) {
		getActiveExam(userId).removeDependentChapter(chapter);
	}
	
	public static void removeQuestion(Long userId){
		getActiveExam(userId).removeQuestionFromQueue();
	}
	public static void addChaptersToQueue(Long userId, List<GraphChapterNode> chapter){
		getActiveExam(userId).addChaptersToQueue(chapter);
	}
	
	public static void addQuestionsToQueue(Long userId, List<Question> questions){
		getActiveExam(userId).addQuestionsToQueue(questions);
	}
	
	public static void addChapterAnswer(Long userId, GraphChapterNode chapter, Answer answer) {
		getActiveExam(userId).addChapterAnswer(chapter, answer);
	}
	
	public static int getGenerateCount(Long userId){
		return getActiveExam(userId).getGenerateCount();
	}
	
	public static ExamResult getExamResult(Long userId){
		return getActiveExam(userId).getResult();
	}
	
	public static List<ChapterResult> getResults(Long userId){
		return getActiveExam(userId).getResult().getChapterResults();
	}
	
	public static GraphChapterNode getNextChapter(Long userId) {
		return getActiveExam(userId).getChapter();
	}
	
	public static Boolean isQuestionQueueEmpty(Long userId) {
		return getActiveExam(userId).isQuestionQueueEmpty();
	}
	
	public static Boolean isChapterQueueEmpty(Long userId) {
		return getActiveExam(userId).isChapterQueueEmpty();
	}
	
	public static Boolean hasDoneDependencies(Long userId, List<GraphChapterNode> chapters){
		return getActiveExam(userId).hasDoneDependencies(chapters);
	}
	
	public static Question getNextQuestion(Long userId) {
		return getActiveExam(userId).getQuestion();
	}
	
	public static Boolean isActive(Long userId){
		return (getActiveExam(userId) != null);
	}
	
	private static ActiveExam getActiveExam(Long userId) {
		for (ActiveExam activeExam : activeExams) if (activeExam.getUser().getId().equals(userId)) return activeExam;
		return null;
	}
}
